linux-kernel.vger.kernel.org archive mirror
 help / color / mirror / Atom feed
* [PATCH v2] x86/earlyprintk: Add a force option for pciserial device
@ 2018-09-28  9:40 Feng Tang
  2018-09-29 16:34 ` Borislav Petkov
  0 siblings, 1 reply; 14+ messages in thread
From: Feng Tang @ 2018-09-28  9:40 UTC (permalink / raw)
  To: Thomas Gleixner, Ingo Molnar, H Peter Anvin, Borislav Petkov,
	Peter Zijlstra, Stuart R . Anderson, alan, x86, linux-kernel
  Cc: Feng Tang

"pciserial" earlyprintk helps much on many modern x86 platforms,
but unfortunately there are still some platforms whose PCI UART
devices have wrong PCI class code, which will be blocked by current
class code check.

Add a option "force" so that developer could still use a UART device
even it has wrong class code, with format "",B:D.F,force,baud". And
the original format ",B:D.F,baud" is kept unchanged.

Suggested-by: Borislav Petkov <bp@alien8.de>
Signed-off-by: Feng Tang <feng.tang@intel.com>
---
 arch/x86/kernel/early_printk.c | 26 ++++++++++++++++++++------
 1 file changed, 20 insertions(+), 6 deletions(-)

diff --git a/arch/x86/kernel/early_printk.c b/arch/x86/kernel/early_printk.c
index 5e801c8..35d0f66 100644
--- a/arch/x86/kernel/early_printk.c
+++ b/arch/x86/kernel/early_printk.c
@@ -213,8 +213,10 @@ static unsigned int mem32_serial_in(unsigned long addr, int offset)
  * early_pci_serial_init()
  *
  * This function is invoked when the early_printk param starts with "pciserial"
  * The rest of the param should be ",B:D.F,baud" where B, D & F describe the
  * location of a PCI device that must be a UART device.
  */
 static __init void early_pci_serial_init(char *s)
 {
@@ -224,6 +226,7 @@ static __init void early_pci_serial_init(char *s)
 	u32 classcode, bar0;
 	u16 cmdreg;
 	char *e;
+	int force = 0;
 
 
 	/*
@@ -252,6 +255,15 @@ static __init void early_pci_serial_init(char *s)
 	if (*s == ',')
 		s++;
 
+	/* User may insist to use a UART device with wrong class code */
+	if (!strncmp(s, "force", 5)) {
+		force = 1;
+		s += 5;
+
+		if (*s == ',')
+			s++;
+	}
+
 	/*
 	 * Second, find the device from the BDF
 	 */
@@ -262,10 +274,12 @@ static __init void early_pci_serial_init(char *s)
 	/*
 	 * Verify it is a UART type device
 	 */
-	if (((classcode >> 16 != PCI_CLASS_COMMUNICATION_MODEM) &&
-	     (classcode >> 16 != PCI_CLASS_COMMUNICATION_SERIAL)) ||
-	   (((classcode >> 8) & 0xff) != 0x02)) /* 16550 I/F at BAR0 */
-		return;
+	if (!force) {
+		if (((classcode >> 16 != PCI_CLASS_COMMUNICATION_MODEM) &&
+		     (classcode >> 16 != PCI_CLASS_COMMUNICATION_SERIAL)) ||
+		   (((classcode >> 8) & 0xff) != 0x02)) /* 16550 I/F at BAR0 */
+			return;
+	}
 
 	/*
 	 * Determine if it is IO or memory mapped
@@ -213,8 +213,10 @@ static unsigned int mem32_serial_in(unsigned long addr, int offset)
  *
  * This function is invoked when the early_printk param starts with "pciserial"
- * The rest of the param should be ",B:D.F,baud" where B, D & F describe the
- * location of a PCI device that must be a UART device.
+ * The rest of the param should be ",B:D.F,baud" or ",B:D.F,force,baud", where
+ * B, D & F describe the location of a PCI device that must be a UART device,
+ * "force" is optional and means insisting using a UART device with a wrong
+ * pci class code.
  */
 static __init void early_pci_serial_init(char *s)
-- 
2.7.4


^ permalink raw reply related	[flat|nested] 14+ messages in thread

end of thread, other threads:[~2018-10-02 15:44 UTC | newest]

Thread overview: 14+ messages (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2018-09-28  9:40 [PATCH v2] x86/earlyprintk: Add a force option for pciserial device Feng Tang
2018-09-29 16:34 ` Borislav Petkov
2018-09-30 13:16   ` Feng Tang
2018-10-01 12:39     ` Borislav Petkov
2018-10-01 14:18       ` Feng Tang
2018-10-01 20:30         ` Borislav Petkov
2018-10-02  9:16           ` Feng Tang
2018-10-02  9:17             ` Thomas Gleixner
2018-10-02  9:48               ` Feng Tang
2018-10-02 10:44                 ` Thomas Gleixner
2018-10-02 12:10                   ` Feng Tang
2018-10-02 15:31                     ` Feng Tang
2018-10-02 15:41                       ` Thomas Gleixner
2018-10-02 15:49                         ` Feng Tang

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).